/* gdt.c */

#include <gdt.h>

/* setup descriptor in gdt */
void gdt_set_gate(int num, unsigned long base, unsigned long lim, unsigned char access, unsigned char gran)
{
	/* set up descriptor base addr */
	gdt[num].base_lo = (base & 0xFFFF);
	gdt[num].base_mid = (base >> 16) & 0xFF;
	gdt[num].base_hi = (base >> 24) & 0xFF;

	/* set up descriptor limits */
	gdt[num].limit_lo = (lim & 0xFFFF);
	gdt[num].granularity = ((lim >> 16) & 0x0F);

	/* set up granularity and access flags */
	gdt[num].granularity |= (gran & 0xF0);
	gdt[num].access = access;
}

/* called by main to set up gdt, first 3 entries in gdt, and call gdt_flush */
void gdt_install()
{
	/* setup gdt pointer and limit */
	gp.limit = (sizeof(struct gdt_entry) * 3) - 1;
	gp.base  = &gdt;

	/* null descriptor */
	gdt_set_gate(0, 0, 0, 0, 0);

	/* cs - base addr is 0, limit is 4gb w/ 4kb gran, 32-bit opcodes */
	gdt_set_gate(1, 0, 0xFFFFFFFF, 0x9A, 0xCF);

	/* ds - same as code, but descriptor type is ds */
	gdt_set_gate(2, 0, 0xFFFFFFFF, 0x92, 0xCF);

	/* flush old gdt and install new */
	gdt_flush();
}
