/* idt.c */

#include <idt.h>
#include <system.h>

/* set up entries in idt - not twiddling gdt to do this */
void idt_set_gate(unsigned char num, unsigned long base, unsigned short sel, unsigned char flags)
{
	/* the interrupt routine base addr */
	idt[num].base_lo = (base & 0xffff);
	idt[num].base_hi = (base >> 16) & 0xffff;

	/* remaining params are straightforward */
	idt[num].sel = sel;
	idt[num].always0 = 0;
	idt[num].flags = flags;
}

/* install idt */
void idt_install()
{
	/* set up idt pointer like in gdt.c */
	idtp.limit = (sizeof(struct idt_entry) * 256) - 1;
	idtp.base = (int)&idt;

	/* clear out idt setting to zeros */
	memset(&idt, 0, sizeof(struct idt_entry) * 256);

	/* point processor internal reg to new idt */
	idt_load();
}
