/* isrs.c */

#include <idt.h>
#include <isrs.h>
#include <screen.h>


/* install the isrs - 0x8e says that entry is present, in ring 0, and has
 * the lower 5 bits set to required '14' (the e in the hex) */
void isrs_install()
{
	idt_set_gate(0, (unsigned)isr0, 0x08, 0x8e);
	idt_set_gate(8, (unsigned)isr8, 0x08, 0x8e);
	idt_set_gate(13, (unsigned)isr13, 0x08, 0x8e);
}

/* an exception string array for simple messaging */
unsigned char *exception_message[] =
{
	"divison by zero",
	"debug",
	"non-maskable interrupt",
	"breakpoint",
	"into detected overflow",
	"out of bounds",
	"invalid opcode",
	"no coprocessor",
	"double fault",
	"coprocessor segment overrun",
	"bad tss",
	"segment not present",
	"general protection fault",
	"page fault",
	"unknown interrupt",
	"coprocessor fault",
	"alignment check",
	"machine check",
	"reserved"
};

/* the fault_handler code referenced in loader.asm */
void fault_handler(struct regs *r)
{
	/* is this a fault 0 to 31 */
	if (r->int_no < 32)
	{
		/* display exception */
		puts(exception_message[r->int_no]);
		puts((unsigned char *)"system halted\n");

		/* simple loop to die */
		for ( ;; )
			;
	}
}
