	[BITS 32]
	[global start]
	[extern main]

mboot:
	;; multiboot macros for readability
	MB_PAGE_ALIGN	equ 1 << 0
	MB_MEM_INFO	equ 1 << 1
	MB_AOUT_KLUDGE	equ 1 << 16
	MB_HEADER_MAGIC	equ 0x1BADB002
	MB_HEADER_FLAGS	equ MB_PAGE_ALIGN | MB_MEM_INFO | MB_AOUT_KLUDGE
	MB_CHECKSUM	equ -(MB_HEADER_MAGIC + MB_HEADER_FLAGS)

	;; grub multiboot header - boot signature
	;; must be 4-byte aligned
	ALIGN 4
multiboot_header:	
	dd MB_HEADER_MAGIC
	dd MB_HEADER_FLAGS
	dd MB_CHECKSUM

	;; aout kludge
	extern	code, bss, end
	dd	mboot
	dd	code
	dd	bss
	dd	end
	dd	start

start:
	mov	esp, _sys_stack	; points stack to new stack area
	call	main		; jumps to c kernel
loop1:
	mov	esi, msg
	call	sprint
	jmp	loop1		; just in case kernel returns

	;; 32-bit video handling
dochar:
	call cprint
sprint:
	mov	eax, [esi]
	lea	esi, [esi+1]
	cmp	al, 0
	jne	dochar
	add	byte [ypos], 1
	mov	byte [xpos], 0
	ret
cprint:
	mov	ah, 0x0F
	mov	ecx, eax
	movzx	eax, byte [ypos]
	mul	edx
	movzx	ebx, byte [xpos]
	shl	ebx, 1
	mov	edi, 0xB8000
	add	edi, eax
	add	edi, ebx
	mov	eax, ecx
	mov	word [ds:edi], ax
	add	byte[xpos], 1
	ret

	;; the gdt (global descriptor table)
	[global gdt_flush]	; allow c code to link this
	[extern gp]		; gp is in gdt.c
gdt_flush:
	lgdt	[gp]
	mov	ax, 0x10	; 0x10 is offset in gdt to ds
	mov	ds, ax
	mov	es, ax
	mov	fs, ax
	mov	gs, ax
	mov	ss, ax
	jmp	0x08:gdt_flush2	; 0x08 is offset to code seg - far jmp	
gdt_flush2:	
	ret			; return back to c code

	;; isr (interrupt service routines)
	[global idt_load]
	[extern idtp]		; in idt.c

idt_load:
	lidt	[idtp]
	ret

	[global isr0]
	[global isr8]
	[global isr13]

	;; exceptions
isr0:
	cli
	push	byte	0
	push	byte	0
	jmp	isr_common_stub

isr8:
	cli
	push	byte	8
	jmp	isr_common_stub

isr13:
	cli
	push	byte	13
	jmp	isr_common_stub

	;; let assembler know that fault_handler is in isrs.c
	[extern fault_handler]

	;; common isr stub saves processor state, sets up kernel mode segs,
	;; calls c-level fault handler, and restores stack frame
isr_common_stub:
	pusha
	push	ds
	push	es
	push	fs
	push	gs
	mov	ax, 0x10	; load kernel ds descriptor
	mov	ds, ax
	mov	es, ax
	mov	fs, ax
	mov	gs, ax
	mov	eax, esp	;push the stack
	push	eax
	mov	eax, fault_handler
	call	eax		; special call preserves eip reg
	pop	eax
	pop	gs
	pop	fs
	pop	es
	pop	ds
	popa
	add	esp, 8		; clean up pushed error code and pushed isr num
	iret			; pops 5 at once: cs, eip, eflags, ss, and esp

	;; some silly debugging helpers
	xpos	db 0
	ypos	db 0
	msg	db "Back in ASM", 0

	;; bss section
	[section .bss]
	resb 0x1000	; reserves 4kb
_sys_stack:
					
