#include <system.h>
#include <screen.h>
#include <string.h>

/* define textpointer */
unsigned short *textmemptr;
int attrib = 0x0f;
int csr_x = 0;
int csr_y = 0;

/* scroll screen */
void scroll(void)
{
	unsigned blank, temp;

	/* a blank is a space - give it back color also */
	blank = 0x20 | (attrib << 8);

	/* if csr_y == 25, scroll */
	if (csr_y >= 25)
	{
		/* move current text chunk back in the buffer by 1 */
		temp = csr_y - 25 + 1;
		memcpy((unsigned char *)textmemptr, textmemptr + temp * 80, (25 - temp) * 80 * 2);

		/* set chunk of memory on last line to blank */
		memsetw(textmemptr + (25 - temp) * 80, blank, 80);
		csr_y = 25 - 1;
	}
}

/* update hw cursor */
void move_csr(void)
{
	unsigned temp;

	/* index = [ (y * width) + x ] */
	temp = csr_y * 80 + csr_x;

 	/* send command to indicies 14 & 15 in crt control register */
  	/* these are high and low bytes of index that show where cursor is */
	/* reference http://www.bracken.com/home/vga/ */
	outportb(0x3D4, 14);
	outportb(0x3D5, temp >> 8);
	outportb(0x3D4, 15);
	outportb(0x3D5, temp);
}

/* clears screen */
void cls(void)
{
	unsigned blank;
	int i;

	blank = 0x20 | (attrib << 8);

	/* sets entire screen to blanks */
	for (i = 0; i < 25; i++)
		memsetw(textmemptr + i * 80, blank, 80);

	/* update virtual cursor - move physical cursor */
	csr_x = 0;
	csr_y = 0;
	move_csr();
}

/* put single char to screen */
void putch(unsigned char c)
{
	unsigned short *where;
	unsigned att = attrib << 8;

	/* handle backspace */
	if (c == 0x08)
	{
		if (csr_x != 0)
			csr_x--;
	}
	/* handle tab */
	else if (c == 0x09)
	{
		csr_x = (csr_x + 8) & ~(8 - 1);
	}
	/* handle <cr> */
	else if (c == '\r')
	{
		csr_x = 0;
	}
	/* handle newline the yucky dos way */
	else if (c == '\n')
	{
		csr_x = 0;
		csr_y++;
	}
	/* if ( c >= space ) printable; */
	else if ( c >= ' ')
	{
		where = textmemptr + (csr_y * 80 + csr_x);
		*where = c | att; /* char and attributes */
		csr_x++;
	}

	/* if cursor is at edge of screen, insert \n */
	if ( csr_x >= 80)
	{
		csr_x = 0;
		csr_y++;
	}

	/* scroll screen if needed and move cursor */
	scroll();
	move_csr();
}

/* uses above routing to output string */
void puts(unsigned char *text)
{
	int i;

	for (i = 0; i < strlen((char *)text); i++)
		putch(text[i]);
}

/* sets fore- and background colors */
void settextcolor(unsigned char fore, unsigned char back)
{
	/* top four bytes are back, bottom four are fore */
	attrib = (back << 4) | (fore & 0x0F);
}

/* set textmode vga pointer then clear screen */
void init_video(void)
{
	textmemptr = (unsigned short *)0xB8000;
}
